"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBasic = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const joi_1 = __importDefault(require("joi"));
const type_1 = require("../type");
const auth_type_1 = require("./auth-type");
const auth_type_verify_1 = require("./auth-type-verify");
class AuthBasic {
    static self;
    fnVerify;
    constructor(fnVerify) {
        this.fnVerify = fnVerify;
    }
    static init(fnVerify) {
        this.self = new AuthBasic(fnVerify);
    }
    async verifyIdToken(authToken, context, hasAbility) {
        const _verifiedIdToken = await this.fnVerify({ ...authToken.authMetadata?.basic, hasAbility, roleType: authToken.info.headers?.roleType });
        const idToken = (0, type_1.validateOrThrowError)(joi_1.default.object().keys(auth_type_verify_1.schemaVerifyIdToken), _verifiedIdToken, { stripUnknown: true }, 'verifyIdToken');
        authToken.type = auth_type_1.AuthTokenType.Basic;
        authToken.grant = true;
        authToken.info.auth = { rules: undefined, ...idToken, ability: undefined };
        authToken.info.requestedBy = idToken.userId;
        authToken.info.requestedByName = idToken.userLogin;
        const ability = (0, auth_type_1.generateAbility)(idToken.rules, { context, accessInfo: authToken.info });
        authToken.info.ability = ability;
        authToken.info.auth.ability = ability;
        return authToken;
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthBasic initialization is required');
    }
}
exports.AuthBasic = AuthBasic;
//# sourceMappingURL=auth-basic.js.map