"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getServiceIdTokenClaim = exports.getServiceIdTokenInfo = exports.getServiceIdTokenInfoString = exports.hasAbility = exports.isAuthenticated = exports.auth = void 0;
const ability_1 = require("@casl/ability");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const http_1 = require("../http");
const auth_basic_1 = require("./auth-basic");
const auth_bearer_1 = require("./auth-bearer");
const auth_id_token_1 = require("./auth-id-token");
const auth_type_1 = require("./auth-type");
async function auth(context, authDirective) {
    const authToken = (0, http_1.getAuthToken)(context);
    if (authToken?.info?.token) {
        if (undefined === authToken.grant) {
            switch (authDirective.type) {
                case auth_type_1.AuthTokenType.Bearer: {
                    const _authToken = await auth_bearer_1.AuthBearer.instance.verifyIdToken(authToken, context, authDirective.flag, authDirective.hasAbility);
                    // Generate the ability into auth token and set into context object
                    (0, http_1.setAuthToken)(context, _authToken);
                    return _authToken;
                }
                case auth_type_1.AuthTokenType.Basic: {
                    const _authToken = await auth_basic_1.AuthBasic.instance.verifyIdToken(authToken, context, authDirective.hasAbility);
                    // Generate the ability into auth token and set into context object
                    (0, http_1.setAuthToken)(context, _authToken);
                    return _authToken;
                }
            }
        }
        if (true === authToken.grant) {
            return authToken;
        }
    }
    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Unauthorized access', { hasToken: undefined !== authToken?.info?.token, spec: authDirective });
}
exports.auth = auth;
function isAuthenticated(authToken) {
    return true === authToken?.grant;
}
exports.isAuthenticated = isAuthenticated;
function hasAbility(authToken, rules) {
    if (!isAuthenticated(authToken) || !rules) {
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Unauthorized access', {
            userId: authToken?.info.requestedBy,
            appDomain: authToken?.info.auth?.appDomain,
            productId: authToken?.info.auth?.productId
        });
    }
    const ability = authToken.info.auth?.ability;
    try {
        if (!Array.isArray(rules)) {
            return ability_1.ForbiddenError.from(ability).throwUnlessCan(rules.action, rules.subject, rules.fields);
        }
        for (const rule of rules) {
            ability_1.ForbiddenError.from(ability).throwUnlessCan(rule.action, rule.subject, rule.fields);
        }
    }
    catch (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.FORBIDDEN, {
            userId: authToken.info.requestedBy,
            appDomain: authToken.info.auth?.appDomain,
            productId: authToken.info.auth?.productId,
            rules
        });
    }
}
exports.hasAbility = hasAbility;
function getServiceIdTokenInfoString() {
    const info = getServiceIdTokenInfo();
    return `${info?.userLogin} (userId: ${info?.userId}, orgId: ${info.orgId}, appDomain: ${info.appDomain}, prodoctId: ${info.productId})`;
}
exports.getServiceIdTokenInfoString = getServiceIdTokenInfoString;
function getServiceIdTokenInfo() {
    const info = auth_bearer_1.AuthBearer.instance.serviceIdTokenInfo;
    const claim = auth_id_token_1.AuthIdToken.decode(info.token);
    return {
        ...info,
        userId: claim.userId,
        userLogin: claim.userLogin
    };
}
exports.getServiceIdTokenInfo = getServiceIdTokenInfo;
function getServiceIdTokenClaim(token) {
    return auth_id_token_1.AuthIdToken.decode(token || auth_bearer_1.AuthBearer.instance.serviceIdTokenInfo.token);
}
exports.getServiceIdTokenClaim = getServiceIdTokenClaim;
//# sourceMappingURL=auth.js.map