"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBdkSet = exports.DomainEventCipher = exports.DEFAULT_IV_LENGTH = exports.DEFAULT_CIPHER_ALGO = void 0;
const moment_1 = __importDefault(require("moment"));
const crypto_1 = __importStar(require("crypto"));
const cm_libs_utils_1 = require("cm-libs-utils");
const cm_libs_dukpt_1 = require("cm-libs-dukpt");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_schema_event_1 = require("cm-schema-event");
const domain_schema_1 = require("./domain-schema");
const config_base_1 = require("../config-base");
exports.DEFAULT_CIPHER_ALGO = 'aes-256-cbc';
exports.DEFAULT_IV_LENGTH = 16;
class DomainEventCipher {
    static self;
    cipherAlgo;
    cipherKeyLength;
    cipherIvLength;
    bdkSet;
    ksn;
    schemaPayload;
    _bdk;
    _encKey;
    bdkThreshold; // Threshold by year
    trsmThreshold; // Threshold by minute
    constructor(bdkSet, ksn, cipherAlgo, ivLength, disableSchemaRegistry = false) {
        this.cipherAlgo = cipherAlgo;
        if (cipherAlgo && bdkSet && ksn) {
            this.cipherKeyLength = this.getCipherKeyLength(this.cipherAlgo);
            this.cipherIvLength = ivLength;
            this.bdkSet = bdkSet;
            this.ksn = new cm_libs_dukpt_1.KsnAes(ksn);
        }
        // Load schema from the central schema registry
        this.schemaPayload = new domain_schema_1.DomainSchema(cm_schema_event_1.Schema.DomainEventPayload, disableSchemaRegistry);
    }
    static init(bdkSet, ksn, cipherAlgo = exports.DEFAULT_CIPHER_ALGO, ivLength = exports.DEFAULT_IV_LENGTH, disableSchemaRegistry = false) {
        this.self = new DomainEventCipher(bdkSet, ksn, cipherAlgo, ivLength, disableSchemaRegistry);
        return this.self;
    }
    static term() {
        this.self = undefined;
    }
    static get instance() {
        return this.self || (this.self = DomainEventCipher.init());
    }
    static get isEnabled() {
        return this.self && this.self.isEnabled;
    }
    get isEnabled() {
        return undefined !== this.ksn;
    }
    async encrypt(buffer) {
        const payload = {
            cipher: {
                version: '1',
                algo: this.isEnabled ? this.cipherAlgo : '',
                ksn: this.isEnabled ? null : Buffer.alloc(0),
                iv: this.isEnabled ? (0, crypto_1.randomBytes)(this.cipherIvLength) : Buffer.alloc(0)
            },
            body: buffer
        };
        if (this.isEnabled) {
            const derivedKey = this.encKey.deriveEncryptionKey();
            const cipher = crypto_1.default.createCipheriv(this.cipherAlgo, derivedKey, payload.cipher.iv);
            payload.cipher.ksn = Buffer.from(this.encKey.ksn.buffer);
            payload.body = Buffer.concat([cipher.update(buffer), cipher.final()]);
            this.encKey.ksn.counter++; // Increase transaction counter
        }
        return this.schemaPayload.encode(payload);
    }
    async decrypt(buffer) {
        const payload = await this.schemaPayload.decode(buffer);
        if (!this.isEnabled) {
            return payload.body; // Encoded message
        }
        const cipher = payload.cipher;
        const bdk = this.getBdk(cipher.ksn);
        const derivedKey = new cm_libs_dukpt_1.DukptAesHost(bdk).deriveEncryptionKey(cipher.ksn);
        const decipher = crypto_1.default.createDecipheriv(cipher.algo, derivedKey, cipher.iv);
        return Buffer.concat([decipher.update(payload.body), decipher.final()]);
    }
    get encKey() {
        const minute = new Date().toISOString().substring(0, 16);
        if (!this._encKey || !this.trsmThreshold || minute !== this.trsmThreshold) {
            this.trsmThreshold = undefined;
            // Set TRSM based on number of minute per year
            const minuteOfYear = (0, moment_1.default)().diff((0, moment_1.default)().startOf('year'), 'minute'); // 525600 minutes per year
            this.ksn.trsmId = minuteOfYear;
            const bdk = this.getBdk(this.ksn.buffer);
            const initialKey = new cm_libs_dukpt_1.DukptAesHost(bdk).deriveInitialKey(this.ksn.buffer);
            this._encKey = new cm_libs_dukpt_1.DukptAesClient(initialKey, Buffer.from(this.ksn.buffer));
            this._encKey.ksn.counter = 0;
            this.trsmThreshold = minute;
        }
        return this._encKey;
    }
    getBdk(ksnBuffer) {
        const year = new Date().toISOString().substring(0, 4);
        if (!this._bdk || !this.bdkThreshold || year !== this.bdkThreshold) {
            this.bdkThreshold = undefined;
            // Set KeySetId based on Year (?? ?? YY YY)
            const ksn = new cm_libs_dukpt_1.KsnAes(ksnBuffer);
            const keySetId = ksn.keySetId.toString('hex').substring(0, 4) + year;
            this._bdk = this.bdkSet[keySetId] || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_FOUND, `BDK (${keySetId}) is not found`, { keySetId });
            this.bdkThreshold = year;
        }
        return this._bdk;
    }
    getCipherKeyLength(algo) {
        return cm_libs_utils_1.CIPHER_KEY_LENGTH[algo] || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_SUPPORTED, `Cipher algo (${algo}) is not supported`, { algo });
    }
}
exports.DomainEventCipher = DomainEventCipher;
const getBdkSet = (key, isOptional, value) => {
    const check = (0, cm_libs_utils_1.allowNullOrEmpty)((value) => {
        return (0, cm_libs_utils_1.checkFormat)((0, cm_libs_utils_1.arrayOf)((value) => {
            return (0, cm_libs_utils_1.checkFormat)({
                keySetId: (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString),
                value: cm_libs_utils_1.checkString
            }, value);
        }), JSON.parse(value));
    }, []);
    const bdkSet = value ? check(value) : config_base_1.ConfigBase.get(key, check);
    if (!isOptional && 0 >= bdkSet.length) {
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_CONFIGURED, 'BDK is required', { key });
    }
    const ret = {};
    for (const { keySetId, value } of bdkSet) {
        ret[keySetId] = Buffer.from(value, 'hex');
    }
    return ret;
};
exports.getBdkSet = getBdkSet;
//# sourceMappingURL=domain-event-cipher.js.map