"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestCache = void 0;
const utils_keyvaluecache_1 = require("@apollo/utils.keyvaluecache");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const correlation_id_1 = __importDefault(require("correlation-id"));
const lodash_1 = __importDefault(require("lodash"));
class RequestCache {
    static self;
    defaultCacheSetting = {
        max: 500,
        // for use with tracking overall storage size
        maxSize: 5000,
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        sizeCalculation: (value, key) => {
            return 1;
        },
        // 30 seconds (in milliseconds)
        ttl: 30 * 1000
    };
    cache;
    constructor() {
        this.cache = new utils_keyvaluecache_1.InMemoryLRUCache(this.defaultCacheSetting);
    }
    static get instance() {
        return this.self || (this.self = new RequestCache());
    }
    async set(newId) {
        const id = correlation_id_1.default.getId();
        if (id) {
            const oldId = await this.cache.get(id);
            oldId ? this.cache.set(id, JSON.stringify(lodash_1.default.merge(JSON.parse(oldId), newId))) : this.cache.set(id, JSON.stringify(newId));
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.WARNING, () => 'correlator no id');
        }
    }
    async get() {
        const id = correlation_id_1.default.getId();
        if (id) {
            return JSON.parse(await this.cache.get(id));
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.WARNING, () => 'correlator no id');
        }
    }
}
exports.RequestCache = RequestCache;
//# sourceMappingURL=request-cache.js.map