import { Schema } from 'avsc';
import { DomainEventMessage } from 'cm-schema-event';
import { DomainEventCipher, DomainEventConsumer, DomainEventHandler, DomainEventProducer, ExceptionHandler, Registry } from '../domain-event';
export type KafkaConsumerConfig = {
    topic: string;
    groupId?: string;
    appendOrdinalIndex?: boolean;
    disableSchemaRegistry?: Registry | boolean | null;
    heartbeatTimeout?: number;
    url?: string;
    cipher?: DomainEventCipher;
    ksn?: string;
    bdkSet?: string;
};
export declare class BaseEventProducer<T, M = any> {
    protected producer: DomainEventProducer<T, M>;
    readonly schemaName: string;
    constructor(topics: string[], schema: Schema | Record<string, any>, metadataSchema?: string | Record<string, any>);
    term(): Promise<void>;
    protected publish(message: DomainEventMessage<T, M>, ignoreError?: boolean, overrides?: any): Promise<import("kafkajs").RecordMetadata[]>;
    get topics(): string[];
    emit(body: T, ignoreError?: boolean, correlationId?: string, correlationTopic?: string, eventId?: string, overrides?: any): Promise<import("kafkajs").RecordMetadata[]>;
}
export declare class BaseEventConsumer<T, M = any> {
    protected consumer: DomainEventConsumer<T, M>;
    constructor(topic: string, appendOrdinalIndex?: boolean);
    term(): Promise<void>;
    subscribe(handler: DomainEventHandler<T, M>, exceptionHandler?: ExceptionHandler<T, M>): Promise<void>;
}
export declare class BaseEventConsumerEx<T, M = any> {
    protected consumer: DomainEventConsumer<T, M>;
    constructor(config: KafkaConsumerConfig);
    term(): Promise<void>;
    subscribe(handler: DomainEventHandler<T, M>, exceptionHandler?: ExceptionHandler<T, M>): Promise<void>;
}
