"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseEventConsumerEx = exports.BaseEventConsumer = exports.BaseEventProducer = void 0;
const cm_libs_utils_1 = require("cm-libs-utils");
const nanoid_1 = require("nanoid");
const domain_event_1 = require("../domain-event");
class BaseEventProducer {
    producer;
    schemaName;
    constructor(topics, schema, metadataSchema) {
        this.schemaName = schema['name'] || '';
        this.producer = (0, domain_event_1.initDomainEventProducer)(schema, metadataSchema, topics);
    }
    async term() {
        if (this.producer) {
            await this.producer.term();
        }
    }
    async publish(message, ignoreError, overrides) {
        try {
            return await this.producer.publish(message, undefined, overrides);
        }
        catch (error) {
            if (!ignoreError) {
                throw error;
            }
        }
    }
    get topics() {
        return this.producer.topics;
    }
    async emit(body, ignoreError, correlationId, correlationTopic, eventId, overrides) {
        const message = {
            eventType: this.schemaName,
            eventId: eventId || (0, nanoid_1.nanoid)(),
            correlationId,
            correlationTopic,
            timestamp: (0, cm_libs_utils_1.timeNow)(),
            body,
            metadata: undefined
        };
        return this.publish(message, ignoreError, overrides);
    }
}
exports.BaseEventProducer = BaseEventProducer;
class BaseEventConsumer {
    consumer;
    constructor(topic, appendOrdinalIndex) {
        this.consumer = (0, domain_event_1.initDomainEventConsumer)(topic, appendOrdinalIndex);
    }
    async term() {
        if (this.consumer) {
            await this.consumer.term();
        }
    }
    async subscribe(handler, exceptionHandler) {
        return this.consumer.subscribe(handler, exceptionHandler);
    }
}
exports.BaseEventConsumer = BaseEventConsumer;
class BaseEventConsumerEx {
    consumer;
    constructor(config) {
        this.consumer = (0, domain_event_1.initDomainEventConsumerEx)(config);
    }
    async term() {
        if (this.consumer) {
            await this.consumer.term();
        }
    }
    async subscribe(handler, exceptionHandler) {
        return this.consumer.subscribe(handler, exceptionHandler);
    }
}
exports.BaseEventConsumerEx = BaseEventConsumerEx;
//# sourceMappingURL=base.event.js.map