"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseServiceEx = exports.BaseService = exports.MutationAction = void 0;
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const joi_1 = __importDefault(require("joi"));
const lodash_1 = __importStar(require("lodash"));
const config_base_1 = require("../config-base");
const misc_1 = require("../misc");
const secure_log_1 = require("../misc/secure-log");
const model_1 = require("../model");
const utils_1 = require("cm-libs-utils/dist/utils");
var MutationAction;
(function (MutationAction) {
    MutationAction["ADD"] = "add";
    MutationAction["EDIT"] = "edit";
    MutationAction["ACTIVATE"] = "activate";
    MutationAction["DEACTIVATE"] = "deactivate";
})(MutationAction || (exports.MutationAction = MutationAction = {}));
class BaseService {
    BASE_SERVICE_LOG = config_base_1.ConfigBase.get('BASE_SERVICE_LOG', (0, utils_1.allowNullOrEmpty)(utils_1.checkBoolean, false));
    BASE_SERVICE_FILTER_ORG = config_base_1.ConfigBase.get('BASE_SERVICE_FILTER_ORG', (0, utils_1.allowNullOrEmpty)(utils_1.checkBoolean, true));
    entityName;
    model;
    schemaBrowse;
    schemaRead;
    schemaEdit;
    schemaAdd;
    schemaDelete;
    schemaActivate;
    schemaDeactivate;
    auditTrail;
    schemaNoInput;
    selective = {};
    constructor({ entityName, model, schemaBrowse, schemaRead, schemaEdit, schemaAdd, schemaDelete, schemaActivate, schemaDeactivate, auditTrail = false }) {
        this.entityName = entityName;
        this.model = model;
        this.schemaBrowse = schemaBrowse;
        this.schemaRead = schemaRead;
        this.schemaEdit = schemaEdit;
        this.schemaAdd = schemaAdd;
        this.schemaDelete = schemaDelete;
        this.schemaActivate = schemaActivate;
        this.schemaDeactivate = schemaDeactivate;
        this.auditTrail = auditTrail;
        this.schemaNoInput = joi_1.default.object().unknown(false);
    }
    /// For Overriding Purpose ///
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    filterOrganization(_params, _accessInfo) { }
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    filterRecordStatus(_params, _accessInfo) { }
    async browse(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        const dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.browse(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    //#region Read
    async read(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        const prepareId = {};
        prepareId[`${lodash_1.default.camelCase(this.entityName)}Id`] = params?.['where'].id;
        await misc_1.RequestCache.instance.set(prepareId);
        const dbRetData = config?.transaction
            ? await config.transaction[(0, lodash_1.camelCase)(this.entityName)].findUnique(params)
            : await this.model.instance.run(accessInfo, () => this.model.instance.model.findUnique(params));
        // const dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.findUnique(params))
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    //#region Edit
    async edit(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? (0, lodash_1.cloneDeep)({ ...params }) : {};
            const actionName = MutationAction.EDIT;
            if (config?.transaction) {
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        }
                    }
                }));
            }
            else {
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        }
                    }
                })));
            }
        }
        else {
            dbRetData = config?.transaction
                ? await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update(params)
                : await this.model.instance.run(accessInfo, () => this.model.instance.model.update(params));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    //#region Add
    async add(params, accessInfo, config) {
        if (this.auditTrail) {
            const tempArgs = params ? (0, lodash_1.cloneDeep)({ ...params }) : {};
            const actionName = MutationAction.ADD;
            params['data'].Timelines = {
                create: {
                    action: actionName,
                    args: { ...tempArgs },
                    ...model_1.OmitCreateInput
                }
            };
        }
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        const dbRetData = config?.transaction
            ? await config.transaction[(0, lodash_1.camelCase)(this.entityName)].create(params)
            : await this.model.instance.run(accessInfo, () => this.model.instance.model.create(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    // #region Delete
    async delete(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        const dbRetData = config?.transaction
            ? await config.transaction[(0, lodash_1.camelCase)(this.entityName)].delete(params)
            : await this.model.instance.run(accessInfo, () => this.model.instance.model.delete(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    async activate(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? (0, lodash_1.cloneDeep)({ ...params }) : {};
            const actionName = MutationAction.ACTIVATE;
            if (config?.transaction) {
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        status: model_1.RecordStatus.ACTIVE
                    }
                }));
            }
            else {
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        status: model_1.RecordStatus.ACTIVE
                    }
                })));
            }
        }
        else {
            if (config?.transaction)
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update({ ...params, data: { status: model_1.RecordStatus.ACTIVE } });
            else
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({ ...params, data: { status: model_1.RecordStatus.ACTIVE } }));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    async deactivate(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? (0, lodash_1.cloneDeep)({ ...params }) : {};
            const actionName = MutationAction.DEACTIVATE;
            if (config?.transaction) {
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        status: model_1.RecordStatus.INACTIVE
                    }
                }));
            }
            else {
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        status: model_1.RecordStatus.INACTIVE
                    }
                })));
            }
        }
        else {
            dbRetData = config?.transaction
                ? await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update({ ...params, data: { status: model_1.RecordStatus.INACTIVE } })
                : await this.model.instance.run(accessInfo, () => this.model.instance.model.update({ ...params, data: { status: model_1.RecordStatus.INACTIVE } }));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
}
exports.BaseService = BaseService;
class BaseServiceEx extends BaseService {
    /// For Overriding Purpose ///
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    filterOrganization(_params, _accessInfo) { }
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    filterRecordStatus(_params, _accessInfo) { }
    async activate(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? (0, lodash_1.cloneDeep)({ ...params }) : {};
            const actionName = MutationAction.ACTIVATE;
            if (config?.transaction) {
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        recordStatus: model_1.RecordStatus.ACTIVE
                    }
                }));
            }
            else {
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        recordStatus: model_1.RecordStatus.ACTIVE
                    }
                })));
            }
        }
        else {
            if (config?.transaction)
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update({ ...params, data: { recordStatus: model_1.RecordStatus.ACTIVE } });
            else
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({ ...params, data: { recordStatus: model_1.RecordStatus.ACTIVE } }));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    async deactivate(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        if (params?.['select'])
            params['select'] = (0, lodash_1.merge)(params['select'], this.selective);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? (0, lodash_1.cloneDeep)({ ...params }) : {};
            const actionName = MutationAction.DEACTIVATE;
            if (config?.transaction) {
                dbRetData = await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        recordStatus: model_1.RecordStatus.INACTIVE
                    }
                }));
            }
            else {
                dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update((0, lodash_1.merge)(params, {
                    data: {
                        Timelines: {
                            create: {
                                action: actionName,
                                args: { ...tempArgs },
                                ...model_1.OmitCreateInput
                            }
                        },
                        recordStatus: model_1.RecordStatus.INACTIVE
                    }
                })));
            }
        }
        else {
            dbRetData = config?.transaction
                ? await config.transaction[(0, lodash_1.camelCase)(this.entityName)].update({ ...params, data: { recordStatus: model_1.RecordStatus.INACTIVE } })
                : await this.model.instance.run(accessInfo, () => this.model.instance.model.update({ ...params, data: { recordStatus: model_1.RecordStatus.INACTIVE } }));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
}
exports.BaseServiceEx = BaseServiceEx;
//# sourceMappingURL=base.service.js.map