"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasAbilityDirective = exports.hasAbilityDirectiveTypeDefs = void 0;
const joi_1 = __importDefault(require("joi"));
const graphql_tag_1 = require("graphql-tag");
const utils_1 = require("@graphql-tools/utils");
const graphql_1 = require("graphql");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const common_1 = require("../common");
exports.hasAbilityDirectiveTypeDefs = (0, graphql_tag_1.gql) `
    scalar JSON

    directive @hasAbility(action: String!, subject: String!, fields: String, conditions: JSON, authType: String, authFlag: Boolean) on FIELD | FIELD_DEFINITION
`;
function hasAbilityDirective(schema, directiveName = 'hasAbility') {
    return (0, utils_1.mapSchema)(schema, {
        [utils_1.MapperKind.OBJECT_FIELD]: (fieldConfig) => {
            const hasAbilityDirective = (0, utils_1.getDirective)(schema, fieldConfig, directiveName)?.[0];
            if (hasAbilityDirective) {
                const { resolve = graphql_1.defaultFieldResolver } = fieldConfig;
                const { error, value } = joi_1.default.object()
                    .keys({
                    action: common_1._ruleAction.required(),
                    subject: common_1._ruleSubject.required(),
                    fields: common_1._ruleFields,
                    conditions: common_1._ruleConditions,
                    authType: common_1._authTokenType,
                    authFlag: common_1._boolean.default(false)
                })
                    .validate(hasAbilityDirective);
                if (error) {
                    if (error instanceof graphql_1.GraphQLError) {
                        throw error;
                    }
                    const { message } = (0, cm_libs_context_aware_error_1.describeError)(error);
                    throw new graphql_1.GraphQLError(message, { originalError: error, extensions: { code: cm_libs_context_aware_error_1.errors.INVALID_REQUEST, originalError: error } });
                }
                fieldConfig.resolve = async (source, args, context, info) => {
                    const rule = {
                        action: value.action,
                        subject: value.subject,
                        fields: value.fields,
                        conditions: value.conditions
                    };
                    try {
                        if (undefined === value.authType) {
                            (0, common_1.hasAbility)((0, common_1.getAuthToken)(context?.req), rule);
                        }
                        else {
                            const authToken = await (0, common_1.auth)(context?.req, {
                                type: value.authType,
                                flag: value.authFlag,
                                hasAbility: rule
                            });
                            (0, common_1.hasAbility)(authToken, rule);
                        }
                        return await resolve(source, args, context, info);
                    }
                    catch (error) {
                        if (error instanceof graphql_1.GraphQLError) {
                            throw error;
                        }
                        const localeCode = common_1.LocaleCode.EN_US;
                        const { code, message } = await common_1.ErrorHandler.translate(error, error.statusCode, localeCode);
                        throw new graphql_1.GraphQLError(message, { originalError: error, extensions: { code, originalError: error } });
                    }
                };
            }
            return fieldConfig;
        }
    });
}
exports.hasAbilityDirective = hasAbilityDirective;
//# sourceMappingURL=has-ability.directive.js.map